from ttws import blacklisted

def test_blacklist():
	assert blacklisted(".git") == True
	assert blacklisted(".gi") == False
	assert blacklisted("git") == False
	assert blacklisted("foo/.svn/bar") == True
	assert blacklisted("foo/svn/bar") == False
	assert blacklisted("foo./svn/bar") == False
	assert blacklisted("foo/.cvs/bar") == True
	assert blacklisted("foo/cvs/bar") == False
	assert blacklisted("foo./cvs/bar") == False
	assert blacklisted("foo/.hg/bar") == True
	assert blacklisted("foo/hg/bar") == False
	assert blacklisted("foo./hg/bar") == False
	assert blacklisted("foo/.bzr/bar") == True
	assert blacklisted("foo/bzr/bar") == False
	assert blacklisted("foo./bzr/bar") == False



#!/usr/bin/env python
"""Trimming of trailing white spaces.

Author-email: "Dietmar Winkler" <dietmar.winkler@dwe.no>

License: See UNLICENSE file

This script will recursively remove all trailing white spaces in all
text files in a given directory. Binary files and files residing in
version control specific directories are skipped.

As an addition one can also let it clean out obsolete or empty/superfluous
Modelica annotations from Modelica (`*.mo`) files.

It uses [Adam Hupp](http://hupp.org/adam)'s
[python-magic](https://github.com/ahupp/python-magic) as binary tester.

As a fallback (especially if libmagic is not available, like on Windows)
it acts only on files with a given file extension listed in 'extstring'.

"""


import os
import textwrap
import io

from pyparsing import (White, Keyword, nestedExpr, lineEnd, Suppress,
                       ZeroOrMore, Optional, ParseException, FollowedBy,
                       CaselessLiteral)


# For Windows: list of file extensions to apply the script on
# (including the dot and separated by a comma)
extstring = ".mo,.mos,.c,.h,.cpp,.txt,.order"

# list of version control directories to ignore
BLACKLIST = ['.bzr', '.cvs', '.git', '.hg', '.svn', '.#']

# convert the string object to a list object
listofexts  = extstring.split(",")

def unknownOption(script_name, args):
    """Warning message for unknown options."""
    warning = """
        UNKNOWN OPTION: "%s"

        Use %s -h [--help] for usage instructions.
        """ % (args,script_name,)
    print(textwrap.dedent(warning))

def unknownDirectory(args):
    """Warning message for unknown directory."""
    warning = """
        WARNING: Ignoring unknown directory: "%s"
        """ % (args)
    print(textwrap.dedent(warning))

def detecttype(filepath):
    """Detect the mime type of the text file."""
    try:
        from magic import Magic
        mime = Magic(mime=True)
        type = mime.from_file(filepath)
        root, ext = os.path.splitext(filepath)
        if ext in '.mo':
            return "mo"
        elif "text/" in type:
            return "text"
        else:
            return type
    except (ImportError, TypeError):
        root, ext = os.path.splitext(filepath)
        if ext in '.mo':
            return "mo"
        elif ext in listofexts:
            return "text"
        else:
            return "unknown"

def trimWhitespace(filepath, eol):
    """Trim trailing white spaces from a given filepath."""
    try:
        with io.open(filepath, "r") as source:
            lines = [line.rstrip() for line in source]
            while len(lines) > 1 and not lines[-1]:
                lines.pop(-1)
        with io.open(filepath, "w", newline="") as target:
            target.write(eol.join(lines) +eol)
    except (UnicodeDecodeError, TypeError) as err:
        print("\nOops! Failing to process file: %s\n"
              "Are you sure it is of pure ASCII or UTF8 encoding?\n"
              "Message: %s\n") % (source.name, err)
        raise

def flatten(arg):
      ret = []
      for item in arg:
        if type(item)==list:
          ret = ret + flatten(item)
        elif type(item)==tuple:
          ret = ret + flatten(list(item))
        else:
          ret.append(item)
      return ret

def skipNonEmptyGraphics(s, loc, tokens):
    flattened =  flatten(tokens.args[0].asList())
    joinedFlattened = ''.join(flattened)
    graphicsPresent = False
    lastGraphics = 'graphics' in flattened[-1]
    extentPresent = 'extent' in joinedFlattened
    extentDefault = 'extent={{-100,-100},{100,100}}' in joinedFlattened
    for substring in flattened:
        if 'graphics' in substring:
            if lastGraphics:
                graphicsPresent = False
            else:
                graphicsPresent = True
    removeGraphics = not graphicsPresent and (not extentPresent or extentDefault)
    if not removeGraphics:
        raise ParseException('graphics defined, skipping...')

def cleanAnnotation(filepath, eol):
    """Clean out the obsolete or superfluous annotations."""
    with io.open(filepath, 'r') as mo_file:
        string = mo_file.read()
        # remove old Modelica 1 'Window(),' and 'Coordsys()' annotations:
        WindowRef = ZeroOrMore(White(' \t')) + (Keyword('Window')|Keyword('Coordsys')) + nestedExpr() + ',' + ZeroOrMore(White(' \t') + lineEnd)
        out = Suppress(WindowRef).transformString(string)
        # special care needs to be taken if the annotation is the last one
        WindowLastRef = Optional(',') + ZeroOrMore(White(' \t')) + (Keyword('Window')|Keyword('Coordsys')) + nestedExpr() + ZeroOrMore(White(' \t') + lineEnd)
        out = Suppress(WindowLastRef).transformString(out)

        # remove empty and superfluous Dymola specific annotations:
        dymolaRef = (ZeroOrMore(White(' \t'))
                     + ((Optional('__Dymola_') + 'experimentSetupOutput')|
                        Keyword('DymolaStoredErrors'))
                     + ~nestedExpr() + ',' + ZeroOrMore(White(' \t')))
        out = Suppress(dymolaRef).transformString(out)
        # special care of the last one again
        lastDymolaRef = (Optional(',') + ZeroOrMore(White(' \t'))
                         + ((Optional('__Dymola_') + 'experimentSetupOutput')|
                            Keyword('DymolaStoredErrors'))
                         + ~nestedExpr() + ZeroOrMore(White(' \t')))
        out = Suppress(lastDymolaRef).transformString(out)

        # remove superfluous annotations with defaults
        defaultRef = ((Keyword('rotation')|Keyword('visible')|Keyword('origin'))
                      + ZeroOrMore(White(' \t')) + '=' + ZeroOrMore(White(' \t'))
                      + (Keyword('0')|Keyword('true')|Keyword('{0,0}'))
                      + ',' + ZeroOrMore(White(' \t')))
        out = Suppress(defaultRef).transformString(out)
        # special rule for initial scale in order to avoid false positives
        iniSRef = (Keyword('initialScale')
                   + ZeroOrMore(White(' \t')) + '=' + ZeroOrMore(White(' \t'))
                   + Keyword('0.1')
                   + ',' + ZeroOrMore(White(' \t')))
        out = Suppress(iniSRef).transformString(out)
        # special care for the last ones again
        lastDefaultRef = (Optional(',')
                          + (Keyword('rotation')|Keyword('visible')|Keyword('origin'))
                          + ZeroOrMore(White(' \t')) + '=' + ZeroOrMore(White(' \t'))
                          + (Keyword('0')|Keyword('true')|Keyword('{0,0}'))
                          + ZeroOrMore(White(' \t')))
        out = Suppress(lastDefaultRef).transformString(out)
        lastIniSRef = (Optional(',') + Keyword('initialScale')
                       + ZeroOrMore(White(' \t')) + '=' + ZeroOrMore(White(' \t'))
                       + Keyword('0.1')
                       + ZeroOrMore(White(' \t')))
        out = Suppress(lastIniSRef).transformString(out)
        # remove empty and superfluous Documentation annotation:
        docRef = (ZeroOrMore(White(' \t'))
                     + (Keyword('Documentation'))
                     + ~nestedExpr() + ',' + ZeroOrMore(White(' \t')))
        out = Suppress(docRef).transformString(out)
        # special care of the last one again
        lastDocRef = (Optional(',') + ZeroOrMore(White(' \t'))
                         + (Keyword('Documentation')) + ~FollowedBy('/')
                         + ~nestedExpr() + ZeroOrMore(White(' \t')))
        out = Suppress(lastDocRef).transformString(out)


        # remove Icon and Diagram annotations that do not contain any graphics
        emptyRef =  ZeroOrMore(White(' \t')) + (Keyword('Icon')|Keyword('Diagram')) + nestedExpr()('args') + ',' + ZeroOrMore(White(' \t') + lineEnd)
        emptyRef.setParseAction(skipNonEmptyGraphics)
        out = Suppress(emptyRef).transformString(out)
        # special care for the last annotation again
        lastEmptyRef =   Optional(',') + ZeroOrMore(White(' \t')) + (Keyword('Icon')|Keyword('Diagram')) + nestedExpr()('args') + ZeroOrMore(White(' \t') + lineEnd)
        lastEmptyRef.setParseAction(skipNonEmptyGraphics)
        out = Suppress(lastEmptyRef).transformString(out)

        # in case we end up with empty annotations remove them too
        AnnotationRef = ZeroOrMore(White(' \t')) + Keyword('annotation') + nestedExpr('(',');',content=' ') + ZeroOrMore(White(' \t') + lineEnd)
        out = Suppress(AnnotationRef).transformString(out)
    with io.open(filepath,'w', newline= eol) as mo_file:
        mo_file.write(out)

def stripDocString(filepath, eol):
    """Strip spaces between string start/end and tag"""
    with io.open(filepath, 'r') as mo_file:
        string = mo_file.read()

        # define expressions to match leading and trailing
        # html tags, and just suppress the leading or trailing whitespace
        opener = White().suppress() + CaselessLiteral("<html>")
        closer = CaselessLiteral("</html>") + White().suppress()

        # define a single expression to match either opener
        # or closer - have to add leaveWhitespace() call so that
        # we catch the leading whitespace in opener
        either = opener|closer
        either.leaveWhitespace()
        out = either.transformString(string)

    with io.open(filepath, 'w', newline=eol) as mo_file:
        mo_file.write(out)



import sys
import os
import getopt
import textwrap

from . import (BLACKLIST, cleanAnnotation, extstring, detecttype,
               stripDocString, trimWhitespace, unknownDirectory,
               unknownOption)

def main(args=None):
    if args is None:
        args = sys.argv

    script_name = args.pop(0)
    script_name = os.path.split(script_name)[1]

    # Look for optional arguments:
    try:
        opts, dirnames = getopt.getopt(args, "hvsbc", ["help","version","strip","blanks","clean",
                                                      "eol="])
    # If unknown option is given trigger the display message:
    except getopt.GetoptError:
        unknownOption(args)
        sys.exit(0)
    # if no dir name is given print the usage message
    if not dirnames:
        usage(args)
        sys.exit(0)

    # If help option is given display help otherwise display warning:
    cleanOpt = False
    stripOpt = False
    eol = os.linesep
    for opt, arg in opts:
        if opt in ("-h","--help"):
            usage(args)
            sys.exit(0)
        elif opt in ("-v","--version"):
            os.system('pip show ttws')
            sys.exit(0)
        elif opt in ("-c","--clean"):
            cleanOpt = True
        elif opt in ("-b","--blanks"):
            os.system('for fn in `find -name "*.mo"`; do cat -s $fn >$fn.1; mv $fn.1 $fn; done')
            sys.exit(0)
        elif opt in ("-s","--strip"):
            stripOpt = True
        elif opt in ("--eol"):
            eol = {
                "CRLF": "\r\n",
                "LF": "\n",
                "CR": "\r"
            }
            eol = eol.get(arg, os.linesep)
        else:
            unknownOption(args)
            sys.exit(0)

    # Walk through the given path and call trim function for text files only:
    for dirname in dirnames:
        if os.path.exists(dirname) is False:
            # Warn about an unknown directory
            unknownDirectory(dirname)
        else:
            for path, dirs, files in os.walk(dirname):
                for directory in dirs:
                    if directory in BLACKLIST:
                        print("skipping version control dir: %s " % directory)
                        dirs.remove(directory)
                for file in files:
                    filepath = os.path.join(path, file)
                    filetype = detecttype(filepath)
                    if filetype is "mo" and cleanOpt is True:
                        print("trimming and cleaning %s" % filepath)
                        trimWhitespace(filepath, eol)
                        cleanAnnotation(filepath, eol)
                    elif filetype is "mo" and stripOpt is True:
                        print("trimming and stripping %s" % filepath)
                        trimWhitespace(filepath, eol)
                        stripDocString(filepath, eol)
                    elif filetype is "mo" or filetype is "text":
                        print("trimming %s" % filepath)
                        trimWhitespace(filepath, eol)
                    else:
                        print("skipping file of type %s: %s" % (filetype, filepath))

def usage(script_name):
    """Help message on usage."""
    message = """
        Usage: ttws [OPTIONS] <directory> [<directory> ...]

         This script will recursively scan all text files in a given list of
         directories and remove all trailing white space in every line as well
         as multiple blank lines at the end of the file. Binary files and files
         residing in '.bzr', '.cvs', '.git', '.hg', '.svn' directories are
         skipped.

         Since the main application is for Modelica projects it expects all files
         to be of type ASCII or UTF8. Otherwise it will throw an exception,
         report the illegal file and terminate.

        Note for Windows users:
         If you do not have libmagic installed, the script will fall back to
         only trim files with the following extensions:
             %s

        Options:
            -h, --help
                displays this help message

            -v, --version
                displays version information

            -s, --strip
                strips leading or trailing white spaces from info or
                revision strings that contain HTML documentation
                (those disturb the proper HTML rendering in 'some' tools)

            --eol=[CRLF|LF|CR]
                Force the line endings to be of type:
                 - CRLF = '\\r\\n' Windows
                 - LF = '\\n' POSIX
                 - CR = '\\r' Mac (pre OSX)
                If empty or not specified it is set to the OS default.
                I.e., on this machine to: %s.

            -c, --clean
                WARNING: USE THIS OPTION AT YOUR OWN RISK AS
                         IT *MIGHT* BREAK YOUR CODE!
                Removes obsolete or superfluous annotation constructs
                from Modelica files.
                Only use this if your code is under version control
                and in combination with a careful code-diff review.

            -b, --blanks
                suppress repeated empty output lines from *.mo files
                (This option should not be run in combination with others.)

        """ % (extstring, repr(os.linesep))
    print(textwrap.dedent(message))

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
"""Test script that displayes the type of a file.

Author-email: "Dietmar Winkler" <dietmarw at gmx de>

License: See UNLICENSE file

This binary tester uses the python magic implementation from
Adam Hupp, http://hupp.org/adam/hg/python-magic

"""

import os
import sys
import textwrap

import magic

mime = magic.Magic(mime=True)

def detecttype(filename):
	"""Detect the mime type of the file"""
	type = mime.from_file(filename)
	if "text/" in type:
		return "text"
	else:
		return type

def usage(argv):
	"""Help message on usage."""
	message = """
		Usage: %s [OPTIONS] <directory> [<directory> ...]

		 Shows recursively which files in a directory are text
		 and which are binary or other formats

		 If no directory is given, the current directory '.'
		 is used as default.

		Options:
			-h, --help
				displays this help message
		""" % (argv[0],)
	print(textwrap.dedent(message))

# warning message for unkown options
def unkownOption(argv):
	"""Warning message for unkown options."""
	warning = """
		UNKNOWN OPTION: "%s"

		Use %s -h [--help] for usage instructions.
		""" % (argv[1],argv[0],)
	print(textwrap.dedent(warning))

def main(argv):
	import getopt
	# Look for optional arguments:
	try:
		opts, dirnames = getopt.getopt(argv[1:], "h", ["help"])
	# Unkown option is given trigger the display message:
	except getopt.GetoptError:
		unkownOption(argv)
		sys.exit(0)
	for o, a in opts:
		if o == "-h" or "--help":
			usage(argv)
			sys.exit(0)
		else:
			unkownOption(argv)
			sys.exit(0)
	# Default directory:
	if not dirnames:
		dirnames = ["."]
	# Walk recursively through the given directories:
	for dirname in dirnames:
		for path, dirs, files in os.walk(dirname):
			for file in files:
				fullname = os.path.join(path, file)
				try:
					print detecttype(fullname),":", repr(fullname)[1: - 1]
				except IOError:
					pass

if __name__ == "__main__":
	sys.exit(main(sys.argv))



"""Setup script to generate an stand-alone executable.

Author-email: "Dietmar Winkler" <dietmar.winkler@dwe.no>

License: See UNLICENSE file

Usage: Run the build process by running the command 'python setup.py build'
       If everything works well you should find a subdirectory in the build
       subdirectory that contains the files needed to run the script

"""

from setuptools import setup, find_packages


CLASSIFIERS = """
Environment :: Console
Intended Audience :: Developers
Operating System :: OS Independent
Programming Language :: Python :: 2
Programming Language :: Python :: 3
""".strip().splitlines()

META = {
    'name': 'ttws',
    'url': 'https://github.com/dietmarw/trimtrailingwhitespaces',
    'version': '0.8.1',
    'description': 'Script to remove trailing whitespaces from textfiles.',
    'classifiers': CLASSIFIERS,
    'license': 'UNLICENSE',
    'author': 'Dietmar Winkler',
    'author_email': 'dietmar.winkler@dwe.no',
    'packages': find_packages(exclude=['test']),
    'entry_points': {
        'console_scripts': 'ttws = ttws.cli:main'
    },
    'platforms': 'Posix; MacOS X; Windows',
    'include_package_data': False,
    'zip_safe': False,
    'install_requires': ['pyparsing', 'python-magic'],
    'extras_require': {
        'testing': ['pytest']
    }
}


if __name__ == '__main__':
    setup(**META)
